<?php

namespace Marketup\PriceHistoryForWoocommerce;

use WC_Product;
use WC_Product_Variable;

class Core {
    private static $instance = null;

    private $plugin_file;

    public Settings $settings;

    public static function instance() {
        if (! self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    private function __construct() {
        $this->settings = new Settings;
    }

    public function init(string $filename) {
        $this->plugin_file = $filename;

        register_activation_hook($this->plugin_file, fn () => Migrations::run_db_delta());

        add_action('admin_menu', [$this->settings, 'add_admin_menu']);
        add_action('admin_init', [$this->settings, 'register_settings']);

        // On change
        add_action('woocommerce_after_product_object_save', [$this, 'on_product_save']);

        // Admin Display
        add_filter('woocommerce_product_data_tabs', [$this, 'add_price_history_tab']);
        add_action('woocommerce_product_data_panels', [$this, 'price_history_tab_content']);
        add_action('woocommerce_product_after_variable_attributes', [$this, 'add_variation_price_history'], 10, 3);

        add_action('wp_ajax_load_price_history', [$this, 'load_price_history_ajax']);

        foreach ($this->settings->lowest_price_message_hooks as $hook) {
            add_action($hook, [$this, 'display_lowest_price_message'], 20);
        }

        foreach ($this->settings->history_graph_hooks as $hook) {
            add_action($hook, [$this, 'display_history_graph'], 30);
        }

        add_shortcode('marketup_lowest_price_history', [$this, 'lowest_price_history_shortcode']);
        add_shortcode('marketup_price_history_graph', [$this, 'price_history_graph_shortcode']);
    }

    public function on_product_save(WC_Product $product) {
        static $saved = [];

        if (in_array($product->get_id(), $saved)) {
            return;
        }

        if ($product instanceof WC_Product_Variable) {
            return;
        }

        $regular_price = $product->get_regular_price();
        $sale_price = $product->is_on_sale() ? $product->get_sale_price() : null;

        if (empty($regular_price)) {
            return;
        }

        (new PriceHistoryEntry($product->get_id(), $regular_price, $sale_price))->save();

        $saved[] = $product->get_id();
    }

    public function add_price_history_tab($tabs) {
        $tabs['price_history'] = [
            'label' => __('Price History', 'marketup-price-history'),
            'target' => 'price_history_data',
            'class' => ['show_if_simple', 'show_if_variable'],
        ];

        return $tabs;
    }

    public function price_history_tab_content() {
        ?>
        <div id="price_history_data" class="panel woocommerce_options_panel hidden">
        <?php
        $price_history = new PriceHistoryTable;
        $price_history->prepare_items();
        $price_history->display();
        ?>
        </div>
        <?php
    }

    public function add_variation_price_history($loop, $variation_data, $variation) {
        ?>
        <div class="variation-price-history">
        <h2><?php _e('Price History', 'marketup-price-history'); ?></h2>
        <?php
        $price_history = new PriceHistoryTable($variation->ID);
        $price_history->prepare_items();
        $price_history->display();
        ?>
        </div>
        <?php
    }

    private function low_prices(WC_Product $product) {
        if ($product instanceof WC_Product_Variable) {
            $ids = $product->get_children();
            if (count($ids) == 0) {
                return;
            }

            $low_prices = PriceHistoryEntry::lowest_prices_in_period_for_variable_products($ids);

            $a = $low_prices['min_sale_price'] && ($low_prices['min_sale_price'] < $low_prices['min_regular_price'])
                ? $low_prices['min_sale_price']
                : $low_prices['min_regular_price'];

            $b = $low_prices['max_sale_price'] && ($low_prices['max_sale_price'] < $low_prices['max_regular_price'])
                ? $low_prices['max_sale_price']
                : $low_prices['max_regular_price'];

            $range = [$a, $b];
            sort($range);

            if ($range[0] == $range[1]) {
                return wc_price($range[0]);
            } else {
                return wc_price($range[0]).' - '.wc_price($range[1]);
            }
        } else {
            $low_prices = PriceHistoryEntry::lowest_prices_in_period($product->get_id());
            if (! $low_prices) {
                return;
            }

            $range_day_low = $low_prices->sale_price > 0
                ? $low_prices->sale_price
                : $low_prices->regular_price;

            return wc_price($range_day_low);
        }
    }

    public function display_lowest_price_message() {
        /** @var WC_Product */
        global $product;

        if (! $product || ! is_a($product, WC_Product::class)) {
            return;
        }

        $low_prices = $this->low_prices($product);
        if ($low_prices === null) {
            return;
        }

        echo '<p class="mu-lowest-recent-price-notice">';
        echo wp_kses_post(
            __(sprintf(
                'Lowest price in the last %d days: <strong>%s</strong>',
                $this->settings->keep_history_in_days,
                $low_prices,
            ), 'marketup-price-history'),
        );
        echo '</p>';
    }

    public function display_history_graph() {
        /** @var WC_Product */
        global $product;

        if (! $product || ! is_a($product, WC_Product::class)) {
            return;
        }

        if ($product->is_type('simple')) {
            $price_history = PriceHistoryEntry::get_price_history($product->get_id(), true);
            if (! $price_history) {
                return;
            }

            $price_history = [$product->get_id() => $price_history];
        } else {
            $price_history = [];
            foreach ($product->get_children() as $child) {
                $price_history[$child] = PriceHistoryEntry::get_price_history($child, true);
            }

            if (count($price_history) == 0) {
                return;
            }
        }

        wp_enqueue_script(
            'mu-price-history-graph',
            plugin_dir_url($this->plugin_file).'assets/dist/chart.js',
            [],
            '1.0.0'
        );

        ?>
        <div class="mu-price-history-graph-wrapper">
            <h4><?php esc_html_e('Price History', 'marketup-price-history') ?></h4>
            <div
                class="mu-price-history-graph"
                data-history="<?php echo esc_attr(json_encode($price_history)) ?>"
                data-price-label="<?php echo esc_attr_e('Price', 'marketup-price-history') ?>"
            ></div>
        </div>
        <?php
    }

    public function load_price_history_ajax() {
        if (! current_user_can('edit_posts')) {
            wp_send_json_error('Insufficient permissions');

            return;
        }

        $product_id = isset($_POST['product_id']) ? intval($_POST['product_id']) : 0;

        if (isset($_POST['paged'])) {
            $_GET['paged'] = intval($_POST['paged']);
        }

        ob_start();
        $table = new PriceHistoryTable($product_id);
        $table->prepare_items();
        $table->display_table();
        $html = ob_get_clean();

        wp_send_json_success(['html' => $html]);
    }
}
