<?php

namespace Marketup\PriceHistoryForWoocommerce;

class PriceHistoryEntry {
    const TABLE_NAME = 'marketup_price_history';

    private int $product_id;

    private string $regular_price;

    private ?string $sale_price;

    public function __construct($product_id, $regular_price, $sale_price) {
        $this->product_id = $product_id;
        $this->regular_price = $regular_price;
        $this->sale_price = $sale_price;
    }

    public function save() {
        global $wpdb;
        $table_name = $wpdb->prefix.self::TABLE_NAME;

        $last_entry = $wpdb->get_row($wpdb->prepare(
            "SELECT regular_price, sale_price
             FROM $table_name
             WHERE product_id = %d
             ORDER BY inserted_at DESC
             LIMIT 1",
            $this->product_id
        ));

        if (! $last_entry ||
            $last_entry->regular_price != $this->regular_price ||
            $last_entry->sale_price != $this->sale_price
        ) {
            $wpdb->insert($table_name, [
                'product_id' => $this->product_id,
                'regular_price' => $this->regular_price,
                'sale_price' => $this->sale_price,
                'currency' => get_woocommerce_currency(),
            ]);
        }
    }

    public static function lowest_prices_in_period($product_id) {
        global $wpdb;
        $table_name = $wpdb->prefix.self::TABLE_NAME;

        $days = Core::instance()->settings->get_keep_history_in_days();

        $query = $wpdb->prepare(
            "SELECT MIN(regular_price) as regular_price, MIN(sale_price) as sale_price
            FROM $table_name
            WHERE
              product_id = %d
              AND inserted_at >= DATE_SUB(NOW(), INTERVAL $days DAY)
            GROUP BY product_id
            ORDER BY sale_price, regular_price ASC
            LIMIT 1
            ",
            $product_id
        );

        return $wpdb->get_row($query);
    }

    public static function lowest_prices_in_period_for_variable_products(array $product_ids) {
        global $wpdb;
        $table_name = $wpdb->prefix.self::TABLE_NAME;

        $id_placeholders = implode(',', array_fill(0, count($product_ids), '%d'));
        $days = Core::instance()->settings->get_keep_history_in_days();

        $query = $wpdb->prepare(
            "SELECT
              MIN(regular_price) as regular_price,
              MIN(sale_price) as sale_price
             FROM $table_name
             WHERE
               product_id IN ($id_placeholders)
               AND inserted_at >= DATE_SUB(NOW(), INTERVAL $days DAY)
             GROUP BY product_id
             ORDER BY sale_price, regular_price ASC
             ",
            $product_ids
        );

        $results = $wpdb->get_results($query, ARRAY_A);

        if (! $results) {
            return;
        }

        $regular_prices = array_column($results, 'regular_price');
        $min_regular_price = min($regular_prices);
        $max_regular_price = max($regular_prices);

        $sale_prices = array_column($results, 'sale_price');
        $min_sale_price = min($sale_prices);
        $max_sale_price = max($sale_prices);

        return [
            'min_regular_price' => $min_regular_price,
            'max_regular_price' => $max_regular_price,
            'min_sale_price' => $min_sale_price,
            'max_sale_price' => $max_sale_price,
        ];
    }

    public static function get_price_history($product_id, $is_limited = false) {
        global $wpdb;
        $table_name = $wpdb->prefix.self::TABLE_NAME;

        $query =
            "SELECT regular_price, sale_price, currency, inserted_at
             FROM $table_name
             WHERE product_id = %d";

        if ($is_limited) {
            $day = Core::instance()->settings->get_keep_history_in_days();
            $query .= " AND inserted_at >= DATE_SUB(NOW(), INTERVAL $day DAY)";
        }

        $query .= ' ORDER BY inserted_at ASC';

        $stmt = $wpdb->prepare($query, $product_id);

        return $wpdb->get_results($stmt);
    }
}
