<?php

namespace Marketup\PriceHistoryForWoocommerce;

use WP_List_Table;

class PriceHistoryTable extends WP_List_Table {
    private $id = null;

    public function __construct($id = null) {
        $this->id = $id;

        parent::__construct([
            'singular' => 'Price History',
            'plural' => 'Price History',
            'ajax' => true,
        ]);
    }

    public function prepare_items() {
        global $wpdb;

        // Check if product ID is set
        $product_id = $this->id ?? (isset($_GET['post']) ? intval($_GET['post']) : 0);
        $this->id = $product_id;

        // Pagination
        $per_page = 15;
        $current_page = $this->get_pagenum();
        $offset = ($current_page - 1) * $per_page;

        // Get total number of records
        $total_items = $wpdb->get_var($wpdb->prepare(
            "SELECT COUNT(*) FROM {$wpdb->prefix}marketup_price_history WHERE product_id = %d",
            $product_id
        ));

        // Retrieve data
        $this->items = $wpdb->get_results($wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}marketup_price_history
             WHERE product_id = %d
             ORDER BY inserted_at DESC
             LIMIT %d OFFSET %d",
            $product_id, $per_page, $offset
        ));

        // Set pagination
        $this->set_pagination_args([
            'total_items' => $total_items,
            'per_page' => $per_page,
            'total_pages' => ceil($total_items / $per_page),
        ]);

        // Set column headers
        $this->_column_headers = [
            $this->get_columns(),
            [],
            [],
        ];
    }

    public function get_columns() {
        return [
            'inserted_at' => __('Date'),
            'regular_price' => __('Regular price', 'woocommerce'),
            'sale_price' => __('Sale price', 'woocommerce'),
            'currency' => __('Currency', 'woocommerce'),
        ];
    }

    public function column_default($item, $column_key) {
        switch ($column_key) {
            case 'inserted_at':
                return date_i18n(get_option('date_format').' '.get_option('time_format'), strtotime($item->inserted_at));
            case 'regular_price':
                return wc_price($item->regular_price);
            case 'sale_price':
                return $item->sale_price ? wc_price($item->sale_price) : '—';
            case 'currency':
                return $item->currency;
            default:
                return print_r($item, true);
        }
    }

    public function display() {
        ?>
        <div class="price-history-table-container">
            <?php $this->display_table(); ?>
        </div>
        <?php
    }

    public function display_table() {
        ?>
        <table class="wp-list-table widefat fixed striped">
            <thead>
                <tr>
                    <?php $this->print_column_headers(); ?>
                </tr>
            </thead>
            <tbody>
                <?php $this->display_rows(); ?>
            </tbody>
        </table>
        <?php $this->display_tablenav('bottom'); ?>
        <script>
        jQuery(document).ready(function($) {
            $('.price-history-table-container .pagination-links a').on('click', function(e) {
                e.preventDefault();
                var page = $(this).attr('href').match(/paged=(\d+)/);
                if (page) {
                    loadPriceHistory(page[1]);
                } else {
                    loadPriceHistory(0);
                }
            });

            function loadPriceHistory(page) {
                var data = {
                    action: 'load_price_history',
                    security: $('#price_history_nonce').val(),
                    product_id: <?php echo json_encode($this->id); ?>,
                    paged: page
                };

                $.post(ajaxurl, data, function(response) {
                    if (response.success) {
                        $('.price-history-table-container').html(response.data.html);
                    } else {
                        console.error('Failed to load price history');
                    }
                });
            }
        });
        </script>
        <?php
    }
}
